/* liblwevent/src/liblwevent/300_fdevent.c
 *
 *  (c)2007, Laurence Withers, <l@lwithers.me.uk>.
 *  Released under the GNU GPLv3. See file COPYING or
 *  http://www.gnu.org/copyleft/gpl.html for details.
*/



/* _lwevent_activate()
 *  Activates the event `ev'. Called with EPOLL_CTL_ADD in `op' when creating a new event in 
 *  lwevent_new() or when reactivating an event in lwevent_reactivate(). Called with EPOLL_CTL_MOD
 *  in lwevent_set_events(). Calls epoll_ctl(), returning -1 on permanent failure.
 */
static int _lwevent_activate(int op, struct lwevent* ev)
{
    struct epoll_event ee;

    ee.events = ev->events;
    ee.data.ptr = ev;

    return TEMP_FAILURE_RETRY( epoll_ctl(lwevent_epoll_fd, op, ev->fd, &ee) );
}



/* lwevent_new()
 *  Sets up a new event structure and registers it with epoll_ctl(EPOLL_CTL_ADD).
 */
struct lwevent* lwevent_new(int fd, int events, lwevent_callback callback)
{
    struct lwevent* ev;

    ev = malloc(sizeof(struct lwevent));
    if(!ev) return 0;
    memset(ev, 0, sizeof(struct lwevent));

    ev->fd = fd;
    ev->events = events;
    ev->callback = callback;

    if(_lwevent_activate(EPOLL_CTL_ADD, ev)) {
        free(ev);
        return 0;
    }

    return ev;
}



/* lwevent_free()
 *  Mark `ev' as invalid, closing its file descriptor and calling the destructor (if there is one).
 *  If the object is already invalid, or is null, do nothing.
 */
void lwevent_free(struct lwevent* ev)
{
    if(!ev || ev->fd == -1) return;

    /* call destructor, close file descriptor, mark as invalid */
    if(ev->dtor) ev->dtor(ev);
    TEMP_FAILURE_RETRY( close(ev->fd) );
    _lwevent_invalid(ev);
}



/* lwevent_deactivate()
 *  Temporarily deactivate `ev'. Calls epoll_ctl(EPOLL_CTL_DEL).
 */
int lwevent_deactivate(struct lwevent* ev)
{
    return TEMP_FAILURE_RETRY( epoll_ctl(lwevent_epoll_fd, EPOLL_CTL_DEL, ev->fd, 0) );
}



/* lwevent_reactivate()
 *  Activate `ev'.
 */
int lwevent_reactivate(struct lwevent* ev)
{
    return _lwevent_activate(EPOLL_CTL_ADD, ev);
}



/*!@}*/
/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
